/*
 * Program G_Clic V0.4 or higher
 * (C) by Benjamin Drieu (e-mail: drieu@bocal.cs.univ-paris8.fr)
 * Nov 26rd 1997
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public Licences as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will entertaining,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Publis License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Ce programme est une demonstration de la librairie Grafik. Son but
 * est de montrer comment utiliser les evenements X11 dans un programme
 * Grafik. Ce petit programme dessine quatre triangles dans une
 * fenetre, et dessine des lignes quand on clique et deplace la
 * souris. De plus, une fonction calcule sur quel triangle on appuie
 * sur le bouton ou on le relache. 
 */

/* Inclusion du header ctype.h en raison de la fonction isdigit */
#include <ctype.h>

/* Inclusion du fichier en-tete de la librairie Grafik */
#include "grafik.h"


/* Cette variable sert a determiner si le bouton de la souris est
   enfonce ou non: a 1 il l'est, a 0 il est relache */
int clicked=0;

/* Ces deux variables servent a dessiner une ligne entre l'ancienne
   coordonnee et l'actuelle */
int oldX=0, oldY=0;

/* Ceci est l'epaisseur du trait */
int epaisseur=1;



/* La fonction dessin est celle qui dessine les quatre triangles sur
   la fenetre. A chaque fois, le triangle sera d'une couleur
   differente. L'utilisation de la macro G_WIDTH et de la macro
   G_HEIGHT permet de connaitre les dimensions de la fenetre. */
void 
dessin () {
  G_changerCouleur ("red");
  G_dessinerTrianglePlein (0,0,G_WIDTH/2,G_HEIGHT/2,0,G_HEIGHT);
  G_changerCouleur ("blue");
  G_dessinerTrianglePlein (0,0,G_WIDTH/2,G_HEIGHT/2,G_WIDTH,0);
  G_changerCouleur ("yellow");
  G_dessinerTrianglePlein (G_WIDTH,0,G_WIDTH/2,G_HEIGHT/2,G_WIDTH,G_HEIGHT);
  G_changerCouleur ("green");
  G_dessinerTrianglePlein (0,G_HEIGHT,G_WIDTH/2,G_HEIGHT/2,G_WIDTH,G_HEIGHT);

  /* Tant qu'a faire, on dessinera en noir apres */
  G_changerCouleur ("Black");
}



/* Cette fonction evalue la couleur sur laquelle se trouve le point de
   coordonnes x,y. Elle retourne une chaine de caracteres statique, ce
   qui est une programmation de porc. */
String
getCouleur (int x, int y)
{
  int quotient=x*G_HEIGHT+y*G_WIDTH;

  if ((float)((float)G_WIDTH/(float)x)>(float)((float)G_HEIGHT/(float)y))
    {
      if (quotient < G_HEIGHT*G_WIDTH)
	return "rouge";
      else
	return "vert";
    }
  else
    {
      if (quotient < G_HEIGHT*G_WIDTH)
	return "bleu";
      else
	return "jaune";
    }
}



/* Cette fonction est le handler de l'evenement "Resize". 
   Elle se contente d'appeler la fonction dessin */
void *
redessiner (G_Fenetre *fenetre, int x, int y, char *str)
{
  dessin ();
  printf ("ON DESSINE\n");
  return NULL;
}



/* Cette fonction est le handler de l'evenement "Motion". Elle
   dessine une ligne entre deux points au cas ou le bouton de la
   souris a ete enfonce. Elle actualise de plus les coordonnes du
   dernier point */
void *
gestionMotion (G_Fenetre *fenetre, int x, int y, char *str)
{
  if (clicked)
    {
      G_dessinerLigne (oldX, oldY, x, y);
      oldX=x;
      oldY=y;
    }
  return NULL;
}



/* Cette fonction est le handler de l'evenement "Keyboard". */
void *
gestionKeyboard (G_Fenetre *fenetre, int x, int y, char *str)
{
  if (isdigit(str[0]))
    {
      epaisseur=str[0]-'0';
      printf ("L'epaisseur est maintenant de %d pixels\n", epaisseur);
      G_changerEpaisseur (epaisseur);
    }
  else if (! strcmp(str, "Escape"))
    {
      G_fermerFenetre (fenetre);
    }
  return NULL;
}



/* Cette fonction est le handler de l'evenement "Bouton Up". Elle se
   contente de changer la valeur de la variable "clicked" et
   d'afficher un petit message pour la forme */
void *
gestionUp (G_Fenetre *fenetre, int x, int y, char *str)
{
  if (clicked)
    {
      clicked=0;
      printf ("Et je relache sur le %s\n", getCouleur(x,y));
    }
  return NULL;
}


/* Cette fonction est le handler de l'evenement "Clic1". Elle
   met a jour les coordonnees du dernier point, et met la variable
   "clicked" a 1. Pour la forme, elle affiche un message pour montrer
   ou on a clique */
void *
gestionClic1 (G_Fenetre *fenetre, int x, int y, char *str)
{
  oldX=x;
  oldY=y;
  clicked=1;
  G_dessinerPoint (x, y);
  printf ("J'ai clique sur le %s\n", getCouleur(x,y));
  return NULL;
}



/* Cette fonction est le handler de l'evenement "Clic3". */
void *
gestionClic3 (G_Fenetre *fenetre, int x, int y, char *str)
{
  /* On redessine le fond */
  dessin ();

  /* Tant qu'a faire, on dessine en noir */
  G_changerCouleur ("Black");

  printf ("On efface tout et on recommence !\n");
  return NULL;
}



/* Enfin, la fonction principale. */
void
main (int argc, char **argv)
{
  /* Initialisation des parametres X. Le parametre passe en argument
     est le display. Par commodite, nous mettons ici le premier
     argument passe a la ligne de commande, ce qui permet de lancer le
     programme sur n'importe quel display: "clic merlin:0.0" */
  G_init (argv[1]);

  /* Ici on ouvre la fenetre, on lui donne un titre et des dimensions
   */ 
  G_ouvrirFenetre ("Exemple de gestion d'evenements", 300, 300);

  /* On specifie quels handlers gereront les evenements Clic1, Motion
     et MouseUp */
  G_ajouterEvenement (G_Clic1, gestionClic1);
  G_ajouterEvenement (G_Clic3, gestionClic3);
  G_ajouterEvenement (G_Keyboard, gestionKeyboard);
  G_ajouterEvenement (G_MouseMove, gestionMotion);
  G_ajouterEvenement (G_MouseUp, gestionUp);
  G_ajouterEvenement (G_Resize, redessiner);

  dessin ();

  /* Il ne reste plus qu'a faire le lancement de  la boucle
     d'interception d'evenements Grafiks. C'est une fonction bloquante,
     qui ne rend pas la main au programme */
  G_executer ();
 
}

