/*
 * Program G_Pendule2 V0.1 or higher
 * (C) by Benjamin Drieu (e-mail: drieu@bocal.cs.univ-paris8.fr)
 * Jan 1st 1998
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public Licences as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will entertaining,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Publis License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <math.h>
#include "grafik.h"

#define MAX 1000
#define POLICE "-adobe-new century schoolbook-medium-r-normal--" \
               "14-140-75-75-p-82-iso8859-1"

void dessin ();
void zPlus ();
void zMoins ();
void keyb ();
void majTexte ();

float vv;
int zoom=10;
int OffsetX=0, OffsetY=0;
float F=0.1;

void 
main (int argc, char **argv)
{
  if (argc==2 && !strcmp(argv[1], "--help"))
    {
      printf ("o Dessin de d'espaces de configurations de pendules "
	      "amortis\n"
	      "\t/\t-> zoom -\n"
	      "\t*\t-> zoom +\n"
	      "\t+\t-> vitesse initiale +\n"
	      "\t-\t-> vitesse initiale -\n"
	      "\tF1\t-> amortissement +\n"
	      "\tF2\t-> amortissement -\n"
	      "\tFleches\t-> deplacement\n"
	      "o 1er Janvier 1998\n\n");
      exit (1);
    }

  G_init ("");
  G_ouvrirFenetre ("Pendule", 600, 300);
  G_ajouterEvenement (G_Keyboard, keyb);

  G_changerPolice (POLICE);
  dessin ();

  G_executer();
}


void
keyb (G_Fenetre *fenetre, int x, int y, String chaine)
{
  if (!strcmp (chaine, "KP_Add"))
    {
      vv+=0.1;
    }
  else if (!strcmp (chaine, "KP_Subtract"))
    {
      vv-=0.1;
    }
  else if (!strcmp (chaine, "Up"))
    {
      OffsetY++;
    }
  else if (!strcmp (chaine, "Down"))
    {
      OffsetY--;
    }
  else if (!strcmp (chaine, "Left"))
    {
      OffsetX++;
    }
  else if (!strcmp (chaine, "Right"))
    {
      OffsetX--;
    }
  else if (!strcmp (chaine, "KP_Multiply"))
    {
      zoom++;
    }
  else if (!strcmp (chaine, "KP_Divide"))
    {
      zoom--;
    }
  else if (!strcmp (chaine, "F1"))
    {
      F+=0.02;
    }
  else if (!strcmp (chaine, "F2"))
    {
      F-=0.02;
    }
  else if (!strcmp (chaine, "Escape"))
    {
      G_quitter ();
    }
  else {
    printf ("%s\n", chaine);
  }

  dessin();
}



void
dessin ()
{
  float teta, v;
  float dt=0.1;
  int compteur;
  float acc;
  float k=1;
  int oldX=0, oldY=0;
  int newX=0, newY=0;

  G_changerCouleur ("PeachPuff");
  G_dessinerRectanglePlein (0, 0, G_WIDTH, G_HEIGHT);
  G_changerCouleur ("maroon");

  v=vv;
  
  teta=0;
  for (compteur=-MAX; compteur<MAX; compteur+=1)
    {
      acc=-k*sin(teta)-F*v;
      teta += v*dt;
      v+=acc*dt;
      newX=G_WIDTH/2+OffsetX*10+teta*zoom;
      newY=G_HEIGHT/2+OffsetY*10+v*zoom;
      if (oldX != 0 || oldY != 0)
	G_dessinerLigne (newX, newY, oldX, oldY); 
      else
	{
	  G_changerEpaisseur (5);
	  G_changerCouleur ("Blue");
	  G_dessinerPoint (newX, newY);
	  G_changerEpaisseur (1);
	  G_changerCouleur ("Maroon");
	}
      oldX=newX;
      oldY=newY;
    }

  majTexte ();

}


void
majTexte ()
{
  char tmp[40];

  sprintf (tmp, "Vitesse initiale: %.1f", vv);
  G_dessinerTexte (tmp, 10, G_HEIGHT-10);

  sprintf (tmp, "Frottement: %.2f", F);
  G_dessinerTexte (tmp, G_WIDTH-G_textLargeur(tmp)-10, G_HEIGHT-10);

}


