/*
 * Program G_Texte V0.2 or higher
 * (C) by Benjamin Drieu (e-mail: drieu@bocal.cs.univ-paris8.fr)
 * Nov 26rd 1997
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public Licences as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will entertaining,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Publis License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/* Inclusion du fichier en-tete de la librairie Grafik */
#include "grafik.h"

#define MAX_FONTS 8
#define MAX_COULEURS 4

int indice=0, indiceCouleurs=0;
int posX=10;
int posY=20;
String couleurs [MAX_COULEURS] = {
  "red1",
  "red2",
  "red3",
  "red4",
};
String fonts [MAX_FONTS] = {
  "-adobe-courier-bold-o-normal--24-240-75-75-m-150-iso8859-1",
  "-adobe-helvetica-medium-o-normal--24-240-75-75-p-130-iso8859-1",
  "-adobe-new century schoolbook-bold-r-normal--24-240-75-75-p-149-iso8859-1",
  "-adobe-symbol-medium-r-normal--24-240-75-75-p-142-adobe-fontspecific",
  "-adobe-times-bold-r-normal--24-240-75-75-p-132-iso8859-1",
  "-b&h-lucida-bold-r-normal-sans-24-240-75-75-p-152-iso8859-1",
  "-b&h-lucidabright-demibold-i-normal--24-240-75-75-p-143-iso8859-1",
  "-bitstream-charter-bold-r-normal--25-180-100-100-p-157-iso8859-1",
};


/* Gestion de l'evenement clavier. Un des choix de la librairie Grafik
   est de considerer les evenements clavier comme des chaines de
   caracteres envoyees au client touche par touche. C'est pourquoi la
   touche Escape est testee grace a la fonction strcmp. Les fonctions
   G_textLargeur et G_textHauteur permettent de deplacer un pointeur
   invisible dans l'application. */
void getKeybrd (G_Fenetre *fenetre, int x, int y, String chaine)
{
  if (! strcmp (chaine, "Escape"))
    {
      G_quitter ();
    }
  else
    {
      G_dessinerTexte (chaine, posX, posY);
      posX+=G_textLargeur(chaine)+10;
      /* Lorsqu'on est pres de la fin de la ligne, on saute une ligne */
      if (posX > (G_WIDTH-10)) 
	{
	  posX=0;
	  posY+=G_textHauteur();
	}
      /* A la fin de la fenetre, il est de bon ton de se remettre au
	 debut, sinon l'exemple n'est pas terrible */
      if (posY > (G_HEIGHT-20))
	{
	  posY=20;
	  G_effacerFenetre();
	}
    }
}


/* Changement de police. La variable indice permet de changer de
   chaine en incrementant un entier */
void
getClic1 (G_Fenetre *fenetre, int x, int y, String chaine)
{
  G_changerPolice (fonts[(++indice)%MAX_FONTS]);
}



/* Changement de police. La variable indice permet de changer de
   chaine en decrementant un entier: changement de police dans l'autre
   sens */
void
getClic2 (G_Fenetre *fenetre, int x, int y, String chaine)
{
  G_changerPolice (fonts[(--indice)%MAX_FONTS]);
}



/* Changement de police. La variable indice permet de changer de
   chaine en decrementant un entier: changement de police dans l'autre
   sens */
void
getResize (G_Fenetre *fenetre, int x, int y, String chaine)
{
  G_effacerFenetre ();
  posX=posY=20;
}



/* Changement de couleur. Les souris du bocal n'ayant que trois
   boutons, je ne fais un balayage des couleurs que dans un sens */ 
void
getClic3 (G_Fenetre *fenetre, int x, int y, String chaine)
{
  G_changerCouleur (couleurs[(++indiceCouleurs)%MAX_COULEURS]);
}



/* Enfin, la fonction principale. */
void
main (int argc, char **argv)
{
  /* Initialisation des parametres X. Le parametre passe en argument
     est le display. Par commodite, nous mettons ici le premier
     argument passe a la ligne de commande, ce qui permet de lancer le
     programme sur n'importe quel display: "clic merlin:0.0" */
  G_init (argv[1]);

  /* Ici on ouvre la fenetre, on lui donne un titre et des dimensions
   */ 
  G_ouvrirFenetre ("Exemple de gestion de clavier", 600, 300);

  /* On met une jolie couleur rouge par defaut */
  G_changerCouleur (couleurs[(indiceCouleurs)]);

  /* On specifie quels handlers gereront les evenements Clic et Keyboard */
  G_ajouterEvenement (G_Keyboard, getKeybrd);
  G_ajouterEvenement (G_Clic1, getClic1);
  G_ajouterEvenement (G_Clic2, getClic2);
  G_ajouterEvenement (G_Clic3, getClic3);
  G_ajouterEvenement (G_Resize, getResize);

  /* Il ne reste plus qu'a faire le lancement de  la boucle
     d'interception d'evenements Grafiks. C'est une fonction
     bloquante, qui ne rend pas la main au programme */  
  G_executer ();
 
}
