/*
 * Library GRAFIK V0.1 or higher
 * (C) by Benjamin Drieu (e-mail: drieu@bocal.cs.univ-paris8.fr)
 * Nov 26rd 1997
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public Licences as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will entertaining,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */




/****************************************/
/*     INCLUSION DES EN-TETES XLIB      */
/****************************************/

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h> 



/****************************************/
/*        DEFINITION DES MACROS         */
/****************************************/

/* Cette macro est le nom de la fenetre par defaut */
#define G_ROOT_WINDOW_NAME "Root Window - Grafik"

/* Pour etre plus propre, chaque evenement pour lequel aucun handler
   n'a ete defini vaudra "G_NO_HANDLER" et non "NULL" */
#define G_NO_HANDLER NULL

/* Par commodite, (re)definition de la valeur de retour d'une fonction
   qui echoue ou qui reussit */
#define G_Erreur 0
#define G_Succes 1

/* Definition de macro-commandes d'acces a la taille de la fenetre
   courante et au repere */
#define G_WIDTH (G_currentFenetre->width)
#define G_HEIGHT (G_currentFenetre->height)
#define G_X0 (G_currentFenetre->x0)
#define G_Y0 (G_currentFenetre->y0)

/* Definition du nombre maximum de fenetres ouvertes simultannement */
#define G_MAX_FENETRES 256

/* Masque d'evenements applicables aux fenetres */
#define G_eventsMask (ExposureMask | ButtonPressMask |\
		      ButtonReleaseMask | KeyPressMask | \
		      PointerMotionMask | StructureNotifyMask)
     
/* Police par defaut */
#define G_DEFAULT_FONT "10x20"
     
/* Conversion degres -> radians (ca doit deja exister, mais j'ai la
   flemme de le verifier) */ 
#define G_toRadian(G_n) ((double) G_n*3.1415926535/360)



/****************************************/
/* DEFINITION DES STRUCTURES DE DONNEES */
/****************************************/

/* Pour une facilite de lecture, definition du type "String". Ses
   proprietes sont les memes que pour un "char *"  */
typedef char * String;

/* Ces defines contiennent tous les numeros lies aux evenements
   Grafiks. Chacun correspond a une entree dans la tables des
   evenements de la fenetre */
#define G_NoEvent       -1
#define G_Clic1         1
#define G_Clic2         2
#define G_Clic3         4
#define G_MouseMove     8
#define G_MouseUp       16
#define G_Keyboard      32
#define G_Expose        64
#define G_Resize        128
#define G_MAX_EVENTS    8
#define G_Clic          (G_Clic1|G_Clic2|G_Clic3)
#define G_Mouse         (G_Clic|G_MouseMove|G_MouseUp)

/* La structure G_Fenetre est "la" structure de la librairie
   Grafik. C'est celle qui contient toutes les informations relatives
   a une fenetre: ses dimensions, sa pixmap de fond, son gc, sa
   Window, et ses evenements Grafik.  La plupart des membres de la
   structure sont des structures definies dans la Xlib, mais les
   evenements sont des pointeurs sur fonction. Dans cette release, les
   evenements sont des fonctions du type "void * fct (G_Fenetre *,
   int, int, String)". Je pense modifier ce type dans une version
   prochaine. */
typedef struct _G_Fenetre
{
  int position;
  int width, height;
  int x0, y0;
  Display *dpy;
  Window win;
  GC gc;
  XGCValues xgcv;
  Font police;
  XFontStruct *fstruct;
  String couleur;
  void * (* events[G_MAX_EVENTS]) (struct _G_Fenetre *, int, int, String);
} G_Fenetre;




/****************************************/
/*     DEFINITION VARIABLES GLOBALES    */
/****************************************/

/* Le tableau G_openFenetres contient un pointeurs sur toutes les
   fenetres ouvertes actuellement. L'entier G_lastFenetre est un
   indice pointant sur la premiere fenetre libre. */
G_Fenetre * G_openFenetres [G_MAX_FENETRES];
int G_lastFenetre;

/* La structure G_currentFenetre est la fenetre courante sur
   laquelle se font toutes les manipulations effectuees par les
   fonctions de dessin les plus basiques. */
G_Fenetre *G_currentFenetre;

/* Cette structure X Display est le display sur lequel on est en train
   de travailler. Cette structure est amenee a disparaitre dans de
   futures releases (c'est laid). */
Display *G_dpy;



/****************************************/
/*             PROTOTYPES               */
/****************************************/

/* Fonctions de gestion des fenetres (fenetre.o) */
G_Fenetre * G_ouvrirFenetre (String, int, int);
void G_changeFenetre (G_Fenetre *);
void G_changerEpaisseur (int);
G_Fenetre * G_xWindowFenetre (Window);
void G_fermerFenetre (G_Fenetre *);
void G_setRepere (int, int);

/* Fonctions de dessin (dessin.o) */
void G_dessinerLigne (int, int, int, int);
void G_dessinerPoint (int, int);
void G_dessiner4Points (int, int);
void G_dessinerTrianglePlein (int, int, int, int, int, int);
void G_dessinerRectanglePlein (int, int, int, int);
void G_dessinerCerclePlein (int, int, int, int);
void G_dessinerTriangle (int, int, int, int, int, int);
void G_dessinerRectangle (int, int, int, int);
void G_dessinerCercle (int, int, int, int);
void G_effacerFenetre ();
void G_dessinerTexte (String, int, int);
int G_changerPolice (String);
int G_textLargeur (String);
int G_textHauteur ();

/* Fonctions de gestion d'evenements (evt.o) */
void G_ajouterEvenement (long, void *);
void G_enleverEvenement (long);
int G_eventType (XEvent *);
int G_executer ();

/* Fonctions ayant trait a l'application (appli.o) */
void G_init (String);
void G_quitter ();
void G_quitApplication (Display *);


/* Fonctions ayant trait a la couleur (couleur.o) */
int G_changerCouleur (String);
int G_changerCouleurRGB (unsigned short, unsigned short, unsigned short);

