#include "grafik.h"

/* Cette fonction ajoute un gestionnaire d'evenement dans le tableau
   des evenements de l'objet fenetre courant (G_currentFenetre) */
void
G_ajouterEvenement (long G_evt, void * G_fct)
{
  int G_indice;
  
  /* Les evenements passes en arguments sont codes sous la forme d'un
     long au lieu d'un int */
  for (G_indice=0; G_indice<=G_MAX_EVENTS; G_indice++)
    {
      if (G_evt & (1<<G_indice))
	{
	  G_currentFenetre->events[G_indice]=G_fct;
	}
    }
}


/* Cette fonction ajoute un gestionnaire d'evenement dans le tableau
   des evenements de l'objet fenetre courant (G_currentFenetre) */
void
G_enleverEvenement (long G_evt)
{
  int G_indice;
  
  /* Les evenements passes en arguments sont codes sous la forme d'un
     long au lieu d'un int */
  for (G_indice=0; G_indice<=G_MAX_EVENTS; G_indice++)
    {
      if (G_evt & (1<<G_indice))
	{
	  G_currentFenetre->events[G_indice]=G_NO_HANDLER;
	}
    }
}



/* Cette fonction convertit un evenement X en un evenement Grafik. Les
   evenements Grafik n'etant pas forcement dans le meme ordre que les
   evenements X, La conversion s'effectue au moyen d'un switch */
int 
G_eventType (XEvent *G_tmpEvent)
{
  int G_tmpNumber;
  int G_indice;

  switch (G_tmpEvent->type)
    {
    case ButtonPress:
      switch (G_tmpEvent->xbutton.button)
	{
	case Button1:
	  G_tmpNumber = G_Clic1;
	  break;
	case Button2:
	  G_tmpNumber = G_Clic2;
	  break;
	case Button3:
	  G_tmpNumber = G_Clic3;
	  break;
	}
      break;

    case MotionNotify:
      G_tmpNumber=G_MouseMove;
      break;

    case ButtonRelease:
      G_tmpNumber = G_MouseUp;
      break;

    case KeyPress:
      G_tmpNumber = G_Keyboard;
      break;

    case Expose:
      G_tmpNumber = G_Expose;
      break;

    case ConfigureNotify:
      G_tmpNumber = G_Resize;
      break;

    default:
      return G_NoEvent;

    }

  for (G_indice=0; G_indice<=G_MAX_EVENTS; G_indice++)
    {
      if (G_tmpNumber & (1<<G_indice))
	{
	  return G_indice;
	}
    }
  
  return G_Erreur;
}



int
G_executer ()
{
  XEvent G_XEvent;
  int G_tmp;
  G_Fenetre * G_eventFenetre;

  while (1)
    {
      G_XEvent.type=G_Erreur;
      XMaskEvent (G_currentFenetre->dpy,
		  G_eventsMask, &G_XEvent); 
      G_eventFenetre=G_xWindowFenetre(G_XEvent.xany.window);
      if (G_eventFenetre == NULL &&
	  !(G_XEvent.type==DestroyNotify ||
	    G_XEvent.type==UnmapNotify))
	{
	  perror ("Warning: Evenement sur fenetre non connue");
	}

      G_tmp=G_eventType(&G_XEvent);
      if (G_tmp != G_NoEvent)
	{
	  if (G_eventFenetre->events[G_tmp]
	      != G_NO_HANDLER) 
	    {
	      switch (1<<G_tmp)
		{
		case G_Clic1:
		case G_Clic2:
		case G_Clic3:
		case G_MouseMove:
		case G_MouseUp:
		  G_eventFenetre->events[G_tmp]
		    (G_eventFenetre, G_XEvent.xbutton.x,
		     G_XEvent.xbutton.y, NULL); 
		  break;
		  
		case G_Keyboard: 
		  G_eventFenetre->events[G_tmp]
		    (G_eventFenetre, G_XEvent.xkey.x, G_XEvent.xkey.y,
		     XKeysymToString(XKeycodeToKeysym(G_eventFenetre->dpy, 
						      G_XEvent.xkey.keycode,0)));
		  break;
		  
		case G_Resize:
		  if (G_eventFenetre->width !=
		      G_XEvent.xconfigure.width ||
		      G_eventFenetre->height !=
		      G_XEvent.xconfigure.height)
		    {
		      G_eventFenetre->width=G_XEvent.xconfigure.width;
		      G_eventFenetre->height=G_XEvent.xconfigure.height;
		      G_eventFenetre->events[G_tmp]
			(G_eventFenetre, G_XEvent.xconfigure.width,
			 G_XEvent.xconfigure.height, NULL);
		    }
		  break;
		  
		default:
		  /* RIEN POUR L'INSTANT !!! */
		  break;
		}
	    }
	}
    }
  /* Ceci ne devrait jamais arriver */
  return G_Succes;
}
