#include "grafik.h"

/*
  Cette fonction est celle qui ouvre une fenetre 
  Il suffit de lui passer en argument le titre de la fenetre (affiche
  en haut sur la barre de la fenetre par les soins du window manager)
  ainsi que les dimensions de la fenetre .
  La fonction fixe la dimension de la fenetre, cree la structure
  Window, cree la pixmap de fond ainsi que le gc, et determine quels
  evenements X seront "ecoutes" par cette fenetre.
  */
G_Fenetre *
G_ouvrirFenetre (String G_name, int G_width, int G_height)
{
  XSetWindowAttributes G_winattr;
  XTextProperty G_tmpTextProperty;
  int G_i;

  /* Verification du nombre de fenetres */
  if (G_lastFenetre == G_MAX_FENETRES)
    {
      perror ("Trop de fenetres ouvertes");
      return NULL;
    }
  
  /* Allocation de la structure fenetre et placement dans le tableau
     des fenetres ouvertes */
  G_currentFenetre = (G_Fenetre *) malloc (sizeof (G_Fenetre));
  G_currentFenetre->position=G_lastFenetre;
  G_openFenetres[G_currentFenetre->position] = G_currentFenetre;
  G_lastFenetre++;

  /* Initialisation du display de la fenetre */
  G_currentFenetre->dpy=G_dpy;
  
  /* Mise a zero des handlers d'evenement */
  for (G_i=0; G_i<G_MAX_EVENTS; G_i++)
    {
      G_currentFenetre->events[G_i]=G_NO_HANDLER;
    }

  /* Fixe les dimensions de la fenetre dans la structure G_fenetre */
  G_currentFenetre->width=G_width;
  G_currentFenetre->height=G_height;

  /* Creation de la Window */
  G_currentFenetre->win = 
    XCreateSimpleWindow(G_currentFenetre->dpy,
			DefaultRootWindow(G_currentFenetre->dpy), 
			0, 0, G_width, G_height, 2, 
			BlackPixel(G_currentFenetre->dpy,
				   DefaultScreen(G_currentFenetre->dpy)), 
			WhitePixel(G_currentFenetre->dpy,
				   DefaultScreen(G_currentFenetre->dpy)));

  /* Les attributs de la fenetres interressants sont le masque
     d'evenement ainsi que le backing store (le serveur X se charge de
     reafficher la fenetre a ma place) */
  G_winattr.event_mask=G_eventsMask;
  G_winattr.backing_store=Always;
  XChangeWindowAttributes (G_currentFenetre->dpy,
			   G_currentFenetre->win, 
			   CWEventMask|CWBackingStore,
			   &G_winattr);

  /* On rajoute un titre */
  XStringListToTextProperty (&G_name, 1, &G_tmpTextProperty);
  XSetWMName (G_currentFenetre->dpy, G_currentFenetre->win,
	      &G_tmpTextProperty); 

  /* Creation du GC */
  G_currentFenetre->xgcv.cap_style=CapRound;
  G_currentFenetre->xgcv.foreground = 
    BlackPixel(G_currentFenetre->dpy,DefaultScreen(G_currentFenetre->dpy)) ;
  G_currentFenetre->xgcv.cap_style=CapButt;
  G_currentFenetre->gc = XCreateGC(G_currentFenetre->dpy,
				   G_currentFenetre->win, 
				   GCForeground|GCCapStyle|GCBackground|GCCapStyle,
				   &G_currentFenetre->xgcv);
  
  /* On alloue de la place pour le nom de la couleur */
  G_currentFenetre->couleur=(String) malloc (256);
  if (G_currentFenetre->couleur == NULL)
    {
      perror ("Memoire insuffisante");
      exit (1);
    }

  G_currentFenetre->fstruct=NULL;
  G_changerPolice (G_DEFAULT_FONT);
  G_currentFenetre->x0=G_currentFenetre->y0=0;
  G_changerEpaisseur (1);

  XMapWindow (G_currentFenetre->dpy, G_currentFenetre->win);

  return G_currentFenetre;

}



void
G_changeFenetre (G_Fenetre * G_tmpFenetre)
{
  G_currentFenetre=G_tmpFenetre;
}


/* Cette fonction modifie l'epaisseur du dessin dans la fenetre */
void
G_changerEpaisseur (int G_width)
{
  G_currentFenetre->xgcv.line_width=G_width;
  XChangeGC (G_currentFenetre->dpy, G_currentFenetre->gc,
	     GCLineWidth, &G_currentFenetre->xgcv);
}



G_Fenetre *
G_xWindowFenetre (Window G_tmpWindow)
{
  int G_i;

  for (G_i=0; G_i < G_lastFenetre; G_i++)
    {
      if (G_openFenetres[G_i]->win == G_tmpWindow)
	{
	  return G_openFenetres[G_i];
	}
    }
  return NULL;  
}



void
G_fermerFenetre (G_Fenetre *G_tmpFenetre)
{
  XFreeGC (G_tmpFenetre->dpy, G_tmpFenetre->gc);
  XDestroyWindow (G_tmpFenetre->dpy, G_tmpFenetre->win);
  
  if (G_lastFenetre == 1)
    {
      G_quitApplication(G_tmpFenetre->dpy);
    }
  else
    {
      G_lastFenetre--;
      G_openFenetres[G_lastFenetre]->position=G_tmpFenetre->position;
      G_openFenetres[G_tmpFenetre->position]=G_openFenetres[G_lastFenetre]; 
    }
}



void 
G_setRepere (int G_x, int G_y)
{
  G_currentFenetre->x0=G_x;
  G_currentFenetre->y0=G_y;
}
